#ifndef _XC3S_H
#define _XC3S_H

#include "Prefix.h"
#include "Memory.h"
#include "XC3S_Register.h"
#include "XC3S_Config.h"
#include "XC3S_Boot.h"
#include "XC3S_PowerReset.h"
#include "XC3S_Status.h"
#include "XC3S_Clock.h"
#include "XC3S_Counter.h"
#include "XC3S_Initiator.h"
#include "XC3S_I2C.h"
#include "XC3S_Protect.h"
#include "PCI_Device.h"
#include <stdio.h>
#include <stdlib.h>

namespace EARTH {

namespace PCI    { class Device; }
namespace Memory { class _; }

namespace XC3S {
	class Register;
	class Config;
	class Boot;
	class PowerReset;
	class Status;
	class Clock;
	class Counter;
	class Initiator;
	class I2C;
	class Protect;

	class _ {
	public:
		struct Argument {
			PCI::Device *Pci;
			Memory::_ *Memory;
		};
	
		_(const Argument *argument) : mPCI(argument->Pci), mMemory(argument->Memory)
		{
			mRegister   = NULL;
			mConfig     = NULL;
			mBoot       = NULL;
			mPowerReset = NULL;
			mStatus     = NULL;
			mClock      = NULL;
			mCounter    = NULL;
			mInitiator  = NULL;
			mI2C        = NULL;
			mProtect    = NULL;

			_ASSERTE(mRegister   == NULL); mRegister   = new class Register  (mPCI           ); _ASSERTE(mRegister  );
			_ASSERTE(mConfig     == NULL); mConfig     = new class Config    (      mRegister); _ASSERTE(mConfig    );
			_ASSERTE(mBoot       == NULL); mBoot       = new class Boot      (      mRegister); _ASSERTE(mBoot      );
			_ASSERTE(mPowerReset == NULL); mPowerReset = new class PowerReset(mPCI, mRegister); _ASSERTE(mPowerReset);
			_ASSERTE(mStatus     == NULL); mStatus     = new class Status    (      mRegister); _ASSERTE(mStatus    );
			_ASSERTE(mClock      == NULL); mClock      = new class Clock     (mPCI, mRegister); _ASSERTE(mClock     );
			_ASSERTE(mCounter    == NULL); mCounter    = new class Counter   (      mRegister); _ASSERTE(mCounter   );
			_ASSERTE(mInitiator  == NULL); mInitiator  = new class Initiator (      mRegister); _ASSERTE(mInitiator );
			_ASSERTE(mI2C        == NULL); mI2C        = new class I2C       (      mRegister); _ASSERTE(mI2C       );
			_ASSERTE(mProtect    == NULL); mProtect    = new class Protect   (      mRegister); _ASSERTE(mProtect   );
		}

		~_()
		{
			delete_(mConfig    );
			delete_(mBoot      );
			delete_(mPowerReset);
			delete_(mStatus    );
			delete_(mClock     );
			delete_(mCounter   );
			delete_(mInitiator );
			delete_(mI2C       );
			delete_(mProtect   );

			delete_(mRegister  );
		}

		void SetVersion(uint version)
		{
			_ASSERTE(mClock);
			mClock->SetVersion(version);
		}

		Register *Register() const { return mRegister; }
		Config *Config() const { return mConfig; }
		Boot *Boot() const { return mBoot; }
		PowerReset *PowerReset() const { return mPowerReset; }
		Status *Status() const { return mStatus; }
		Clock *Clock() const { return mClock; }
		Counter *Counter() const { return mCounter; }
		Initiator *Initiator() const { return mInitiator; }
		I2C *I2C() const { return mI2C; }
		Protect *Protect() const { return mProtect; }

	protected:
		PCI::Device *const mPCI;
		Memory::_ *const mMemory;
		class Register *mRegister;
		class Config *mConfig;
		class Boot *mBoot;
		class PowerReset *mPowerReset;
		class Status *mStatus;
		class Clock *mClock;
		class Counter *mCounter;
		class Initiator *mInitiator;
		class I2C *mI2C;
		class Protect *mProtect;
};

}
}

#endif
