#ifndef _XC3S_Protect_H
#define _XC3S_Protect_H

#include "Prefix.h"
#include "XC3S_Register.h"
#include "EARTH_PT.h"
#include "OS_Thread.h"
#include <stdio.h>

namespace EARTH {
namespace XC3S {
	class Register;

	class Protect {
	public:
		Protect(XC3S::Register *register_) : mRegister(register_)
		{
			_ASSERTE(mRegister);

			mId = 0;
			mUnlocked = false;
		}

		~Protect()
		{
		}

		status Run()
		{
			mId = 0;
			uint sync, data;

			bool b = false;
			for (uint i=0; i<57; i++) {
				Read(&sync);
				if (sync) {
					b = true;
					break;
				}
				mRegister->Write4(A_WRITE, BIT_1(1) << BIT_WRITE_PULSE);
			}
			_ASSERTE(b);

			for (uint i=0; i<57; i++) {
				Read(NULL, &data);
				mId |= static_cast<ulong>(data) << i;

				mRegister->Write4(A_WRITE, BIT_1(1) << BIT_WRITE_PULSE);
			}

			mRegister->Write4(A_WRITE, BIT_1(1) << BIT_WRITE_PULSE);

			for (uint i=0; i<3; i++) {
				Read(NULL, NULL, &mUnlocked);
				if (mUnlocked) break;
				OS::Thread::Sleep((1)*1000*1000);
			}

			if (mUnlocked == false) {
				#if defined(_DEBUG)
//					::wprintf(L"veNgł܂łB\n");
					return PT::STATUS_OK;
				#else
					return PT::STATUS_INTERNAL_ERROR;
				#endif
			}

			return PT::STATUS_OK;
			
			//_ASSERTE(mUnlocked);
		}

		status GetId(ulong *id) const
		{
			if (id == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			*id = mId;

			return PT::STATUS_OK;
		}

		status GetUnlocked(bool *unlocked) const
		{
			if (unlocked == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			*unlocked = mUnlocked;

			return PT::STATUS_OK;
		}

	protected:
		enum {
			A_WRITE = 0x00,
			A_READ  = 0x00
		};

		enum {
			BIT_WRITE_PULSE = 3
		};

		enum {
			BIT_READ_SYNC = 29,
			BIT_READ_DATA,
			BIT_READ_UNLOCK
		};

		XC3S::Register *const mRegister;
		ulong mId;
		bool mUnlocked;

		void Wait()
		{
			mRegister->Write4(A_WRITE, 0);
		}

		void Read(uint *sync, uint *data = NULL, bool *unlock = NULL)
		{
			uint data_;

//			for (uint i=0; i<2; i++) {
				mRegister->Read4(A_READ, &data_);
				mRegister->Read4(A_READ, &data_);
//			}

			if (sync  ) *sync   = BIT_SHIFT_MASK(data_, BIT_READ_SYNC  , 1);
			if (data  ) *data   = BIT_SHIFT_MASK(data_, BIT_READ_DATA  , 1);
			if (unlock) *unlock = BIT_SHIFT_MASK(data_, BIT_READ_UNLOCK, 1);
		}
	};
}
}

#endif
