#ifndef _XC3S_Register_H
#define _XC3S_Register_H

#include "Prefix.h"
#include "PCI_Device.h"

namespace EARTH {
namespace XC3S {
	class _;

	class Register {
	public:
		Register(PCI::Device *device) : mDevice(device)
		{
			_ASSERTE(mDevice);

			mWrite = false;
			mRead = false;
		}

		~Register()
		{
		}

		void EnableWrite()
		{
			mWrite = true;
		}

		void EnableRead()
		{
			mRead = true;
		}

		status WriteConfig1(uint address, byte data)
		{
			status status = mDevice->WriteConfig(address, &data, sizeof(data));
			return status;
		}

		status WriteConfig2(uint address, ushort data)
		{
			status status = mDevice->WriteConfig(address, &data, sizeof(data));
			return status;
		}

		status WriteConfig4(uint address, uint data)
		{
			status status = mDevice->WriteConfig(address, &data, sizeof(data));
			return status;
		}

		status ReadConfig1 (uint address, byte *data)
		{
			_ASSERTE(data);
			status status = mDevice->ReadConfig(address, data, sizeof(*data));
			return status;
		}

		status ReadConfig2 (uint address, ushort *data)
		{
			_ASSERTE(data);
			status status = mDevice->ReadConfig(address, data, sizeof(*data));
			return status;
		}

		status ReadConfig4 (uint address, uint *data)
		{
			_ASSERTE(data);
			status status = mDevice->ReadConfig(address, data, sizeof(*data));
			return status;
		}

		void Write1(uint address, byte data)
		{
			_ASSERTE(mWrite);

			mDevice->WriteMemory(address*4, &data, sizeof(data));
		}

		void Write2(uint address, ushort data)
		{
			_ASSERTE(mWrite);

			mDevice->WriteMemory(address*4, &data, sizeof(data));
		}

		void Write4(uint address, uint data)
		{
			_ASSERTE(mWrite);

			mDevice->WriteMemory(address*4, &data, sizeof(data));
		}

		void Read1(uint address, byte *data)
		{
			_ASSERTE(mRead);

			_ASSERTE(data);
			mDevice->ReadMemory(address*4, data, sizeof(*data));
		}

		void Read2(uint address, ushort *data)
		{
			_ASSERTE(mRead);

			_ASSERTE(data);
			mDevice->ReadMemory(address*4, data, sizeof(*data));
		}

		void Read4(uint address, uint *data)
		{
			_ASSERTE(mRead);

			_ASSERTE(data);
			mDevice->ReadMemory(address*4, data, sizeof(*data));
		}

	protected:
		PCI::Device *const mDevice;
		bool mWrite, mRead;
	};
}
}

#endif
