#ifndef _XC3S_Status_H
#define _XC3S_Status_H

#include "Prefix.h"
#include "XC3S_Register.h"
#include "EARTH_PT.h"

namespace EARTH {
namespace XC3S {
	class Register;

	class Status {
	public:
		struct Info {
			bool RamOverflow,
				 InitiatorError,
				 InitiatorWarning,
				 InitiatorBusy;
		};

		Status(XC3S::Register *register_) : mRegister(register_)
		{
			_ASSERTE(mRegister);
		}

		~Status()
		{
		}

		void GetInfo(Info *info)
		{
			uint data;

			mRegister->Read4(A_READ, &data);

			_ASSERTE(info);
			info->RamOverflow      = BIT_SHIFT_MASK(data, BIT_RAM_OVERFLOW     , 1);
			info->InitiatorError   = BIT_SHIFT_MASK(data, BIT_INITIATOR_ERROR  , 1);
			info->InitiatorWarning = BIT_SHIFT_MASK(data, BIT_INITIATOR_WARNING, 1);
			info->InitiatorBusy    = BIT_SHIFT_MASK(data, BIT_INITIATOR_BUSY   , 1);
		}

		status GetTransferInfo(PT::Device::TransferInfo *transferInfo)
		{
			if (transferInfo == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			Info info;
			GetInfo(&info);

			transferInfo->TransferCounter0 = info.InitiatorError;
			transferInfo->TransferCounter1 = info.InitiatorWarning;
			transferInfo->BufferOverflow   = info.RamOverflow;

			return PT::STATUS_OK;
		}

	protected:
		enum {
			A_READ = 0x00
		};

		enum {
			BIT_RAM_OVERFLOW = 3,
			BIT_INITIATOR_ERROR,
			BIT_INITIATOR_WARNING,
			BIT_INITIATOR_BUSY
		};

		XC3S::Register *const mRegister;
	};
}
}

#endif
