#ifndef _PCI__H
#define _PCI__H

#include "PCI.h"
#include "PCI_Bus.h"
#include <windows.h>
#include "windrvr.h"

namespace EARTH {
namespace PCI {
	namespace _32 { status NewBus(Bus **); }
	namespace _64 { status NewBus(Bus **); }

	class Device;
	namespace _32 { status NewDevice(HANDLE, const Target *, Device **); }
	namespace _64 { status NewDevice(HANDLE, const Target *, Device **); }

	class __ : public _ {
	public:
		__(uint kernelBit) : mKernelBit(kernelBit)
		{
			mBus = NULL;
		}

		virtual ~__()
		{
			delete_(mBus);
		}

		virtual status Init()
		{
			_ASSERTE(mBus == NULL);

			status status;

			if (mKernelBit == 32) {
				status = _32::NewBus(&mBus);
			} else {
				status = _64::NewBus(&mBus);
			}

			return status;
		}

		virtual class Bus *Bus() const
		{
			return mBus;
		}

		virtual status NewDevice(const Target *target, Device **device)
		{
			_ASSERTE(target);

			_ASSERTE(mBus);
			HANDLE handle = mBus->Handle();
			if (handle == INVALID_HANDLE_VALUE) return PT::STATUS_INTERNAL_ERROR;

			status status;

			if (mKernelBit == 32) {
				status = _32::NewDevice(handle, target, device);
			} else {
				status = _64::NewDevice(handle, target, device);
			}

			return status;
		}

	protected:
		const uint mKernelBit;
		class Bus *mBus;
	};
}
}

#endif
